@@ -26,33 +26,47 @@
 
 #include "tracker-miner-applications-meego.h"
 
-/* The meego desktop files are using the qt translation system to get
- * localized strings using catalog and string ids. QApplication and
- * MLocale are needed for loading the translation catalogs. The
- * returned string is a multi-string one which has parts of different
- * lenght separated by '\x9C' unicode escape sequences.
- *
- * FIXME: This is insane, try to get rid of at least some of the extra
- * layers here.
- */
-gchar *
-tracker_miner_applications_meego_translate (const gchar *catalogue,
-					    const gchar *id)
+static QCoreApplication *app = NULL;
+
+void
+tracker_miner_applications_meego_init (void)
 {
 	char *argv[] = { "dummy", NULL };
 	int argc = 1;
 
 	/* We need the app for loading translations */
-	QApplication app (argc, argv, FALSE);
+	app = new QApplication (argc, argv, FALSE);
+}
 
+void
+tracker_miner_applications_meego_shutdown (void)
+{
+	delete app;
+}
+
+/* The meego desktop files are using the qt translation system to get
+ * localized strings using catalog and string ids. QApplication and
+ * MLocale are needed for loading the translation catalogs. The
+ * returned string is a multi-string one which has parts of different
+ * length separated by '\x9C' unicode escape sequences.
+ *
+ * FIXME: This is insane, try to get rid of at least some of the extra
+ * layers here.
+ */
+gchar *
+tracker_miner_applications_meego_translate (const gchar  *catalogue,
+                                            const gchar  *id)
+{
 	/* Get the system default locale */
 	MLocale locale;
 
-	/* Load the catalog from disk */
-	locale.installTrCatalog (catalogue);
-	MLocale::setDefault (locale);
+	/* Load the catalog from disk if not already there */
+	if(!locale.isInstalledTrCatalog (catalogue)) {
+		locale.installTrCatalog (catalogue);
+		MLocale::setDefault (locale);
+	}
 
-	gchar *ret = g_strdup (qtTrId (id). toUtf8 ().data ());
+	gchar *ret = g_strdup (qtTrId (id).toUtf8 ().data ());
 
 	/* We only want the first string of the multi-string, so if
 	 * the separator character is found (encoded as C2:9C in UTF-8),
@@ -64,3 +78,12 @@ tracker_miner_applications_meego_translate (const gchar *catalogue,
 
 	return ret;
 }
+
+gchar *
+tracker_miner_applications_meego_get_locale (void)
+{
+	/* Get the system default locale */
+	MLocale locale;
+
+	return g_strdup (locale.name ().toAscii ().data ());
+}
