@@ -60,6 +60,10 @@ typedef struct {
 	const gchar *state;
 	const gchar *address;
 	const gchar *country;
+	const gchar *gps_altitude;
+	const gchar *gps_latitude;
+	const gchar *gps_longitude;
+	const gchar *gps_direction;
 } MergeData;
 
 typedef struct {
@@ -264,6 +268,7 @@ extract_tiff (const gchar          *uri,
 	glong exif_offset;
 	GPtrArray *keywords;
 	guint i;
+	GString *where = NULL;
 
 #ifdef HAVE_LIBIPTCDATA
 	gchar *iptc_offset;
@@ -372,6 +377,12 @@ extract_tiff (const gchar          *uri,
 	md.state = tracker_coalesce_strip (2, xd->state, id->state);
 	md.address = tracker_coalesce_strip (2, xd->address, id->sublocation);
 	md.country = tracker_coalesce_strip (2, xd->country, id->country_name);
+
+	/* FIXME We are not handling the altitude ref here for xmp */
+	md.gps_altitude = tracker_coalesce_strip (2, xd->gps_altitude, ed->gps_altitude);
+	md.gps_latitude = tracker_coalesce_strip (2, xd->gps_latitude, ed->gps_latitude);
+	md.gps_longitude = tracker_coalesce_strip (2, xd->gps_longitude, ed->gps_longitude);
+	md.gps_direction = tracker_coalesce_strip (2, xd->gps_direction, ed->gps_direction);
 	md.creator = tracker_coalesce_strip (3, xd->creator, id->byline, id->credit);
 	md.x_dimension = tracker_coalesce_strip (2, td.width, ed->x_dimension);
 	md.y_dimension = tracker_coalesce_strip (2, td.length, ed->y_dimension);
@@ -468,51 +479,75 @@ extract_tiff (const gchar          *uri,
 		tracker_sparql_builder_object_unvalidated (metadata, xd->license);
 	}
 
-	if (md.city || md.state || md.address || md.country) {
-		gchar *addruri;
+	if (md.address || md.state || md.country || md.city ||
+	    md.gps_altitude || md.gps_latitude || md.gps_longitude) {
 
 		tracker_sparql_builder_predicate (metadata, "slo:location");
 
-		tracker_sparql_builder_object_blank_open (metadata);
+		tracker_sparql_builder_object_blank_open (metadata); /* GeoPoint */
 		tracker_sparql_builder_predicate (metadata, "a");
-		tracker_sparql_builder_object (metadata, "slo:GeoLocation"); /* GeoLocation */
+		tracker_sparql_builder_object (metadata, "slo:GeoLocation");
 
-		addruri = tracker_sparql_get_uuid_urn ();
+		if (md.address || md.state || md.country || md.city) {
+			gchar *addruri;
 
-		tracker_sparql_builder_predicate (metadata, "slo:postalAddress");
-		tracker_sparql_builder_object_iri (metadata, addruri);
+			addruri = tracker_sparql_get_uuid_urn ();
+
+			tracker_sparql_builder_predicate (metadata, "slo:postalAddress");
+			tracker_sparql_builder_object_iri (metadata, addruri);
+
+			tracker_sparql_builder_insert_open (preupdate, NULL);
+			tracker_sparql_builder_subject_iri (preupdate, addruri);
+
+			g_free (addruri);
+
+			tracker_sparql_builder_predicate (preupdate, "a");
+			tracker_sparql_builder_object (preupdate, "nco:PostalAddress");
+
+			if (md.address) {
+				tracker_sparql_builder_predicate (preupdate, "nco:streetAddress");
+				tracker_sparql_builder_object_unvalidated (preupdate, md.address);
+			}
+
+			if (md.state) {
+				tracker_sparql_builder_predicate (preupdate, "nco:region");
+				tracker_sparql_builder_object_unvalidated (preupdate, md.state);
+			}
+
+			if (md.city) {
+				tracker_sparql_builder_predicate (preupdate, "nco:locality");
+				tracker_sparql_builder_object_unvalidated (preupdate, md.city);
+			}
+
+			if (md.country) {
+				tracker_sparql_builder_predicate (preupdate, "nco:country");
+				tracker_sparql_builder_object_unvalidated (preupdate, md.country);
+			}
+
+			tracker_sparql_builder_insert_close (preupdate);
+		}
+
+		if (md.gps_altitude) {
+			tracker_sparql_builder_predicate (metadata, "slo:altitude");
+			tracker_sparql_builder_object_unvalidated (metadata, md.gps_altitude);
+		}
+
+		if (md.gps_latitude) {
+			tracker_sparql_builder_predicate (metadata, "slo:latitude");
+			tracker_sparql_builder_object_unvalidated (metadata, md.gps_latitude);
+		}
+
+		if (md.gps_longitude) {
+			tracker_sparql_builder_predicate (metadata, "slo:longitude");
+			tracker_sparql_builder_object_unvalidated (metadata, md.gps_longitude);
+		}
 
 		tracker_sparql_builder_object_blank_close (metadata); /* GeoLocation */
+	}
 
-		tracker_sparql_builder_insert_open (preupdate, NULL);
-		tracker_sparql_builder_subject_iri (preupdate, addruri);
-
-		g_free (addruri);
-
-		tracker_sparql_builder_predicate (preupdate, "a");
-		tracker_sparql_builder_object (preupdate, "nco:PostalAddress");
-
-		if (md.address) {
-			tracker_sparql_builder_predicate (preupdate, "nco:streetAddress");
-			tracker_sparql_builder_object_unvalidated (preupdate, md.address);
-		}
-
-		if (md.state) {
-			tracker_sparql_builder_predicate (preupdate, "nco:region");
-			tracker_sparql_builder_object_unvalidated (preupdate, md.state);
-		}
-
-		if (md.city) {
-			tracker_sparql_builder_predicate (preupdate, "nco:locality");
-			tracker_sparql_builder_object_unvalidated (preupdate, md.city);
-		}
-
-		if (md.country) {
-			tracker_sparql_builder_predicate (preupdate, "nco:country");
-			tracker_sparql_builder_object_unvalidated (preupdate, md.country);
-		}
-
-		tracker_sparql_builder_insert_close (preupdate);
+	if (md.gps_direction) {
+		tracker_sparql_builder_predicate (metadata, "nfo:heading");
+		tracker_sparql_builder_object_unvalidated (metadata, md.gps_direction);
 	}
 
 	if (id->contact) {
@@ -536,18 +571,35 @@ extract_tiff (const gchar          *uri,
 	}
 
 	for (i = 0; i < keywords->len; i++) {
-		gchar *p;
+		gchar *p, *escaped, *var;
 
 		p = g_ptr_array_index (keywords, i);
+		escaped = tracker_sparql_escape_string (p);
+		var = g_strdup_printf ("tag%d", i + 1);
 
+		/* ensure tag with specified label exists */
+		tracker_sparql_builder_append (preupdate,
+		                               "INSERT { _:tag a nao:Tag ; nao:prefLabel \"");
+		tracker_sparql_builder_append (preupdate, escaped);
+		tracker_sparql_builder_append (preupdate,
+		                               "\" }\nWHERE { FILTER (NOT EXISTS { "
+		                               "?tag a nao:Tag ; nao:prefLabel \"");
+		tracker_sparql_builder_append (preupdate, escaped);
+		tracker_sparql_builder_append (preupdate,
+		                               "\" }) }\n");
+
+		/* associate file with tag */
 		tracker_sparql_builder_predicate (metadata, "nao:hasTag");
+		tracker_sparql_builder_object_variable (metadata, var);
 
-		tracker_sparql_builder_object_blank_open (metadata);
-		tracker_sparql_builder_predicate (metadata, "a");
-		tracker_sparql_builder_object (metadata, "nao:Tag");
-		tracker_sparql_builder_predicate (metadata, "nao:prefLabel");
-		tracker_sparql_builder_object_unvalidated (metadata, p);
-		tracker_sparql_builder_object_blank_close (metadata);
+		if (where == NULL) {
+			where = g_string_new ("} } WHERE { {\n");
+		}
+
+		g_string_append_printf (where, "?%s a nao:Tag ; nao:prefLabel \"%s\" .\n", var, escaped);
+
+		g_free (var);
+		g_free (escaped);
 		g_free (p);
 	}
 	g_ptr_array_free (keywords, TRUE);
@@ -696,7 +748,7 @@ extract_tiff (const gchar          *uri,
 	if (ed->x_resolution) {
 		gdouble value;
 
-		value = ed->resolution_unit == 1 ? g_strtod (ed->x_resolution, NULL) : g_strtod (ed->x_resolution, NULL) * CM_TO_INCH;
+		value = ed->resolution_unit != 3 ? g_strtod (ed->x_resolution, NULL) : g_strtod (ed->x_resolution, NULL) * CM_TO_INCH;
 		tracker_sparql_builder_predicate (metadata, "nfo:horizontalResolution");
 		tracker_sparql_builder_object_double (metadata, value);
 	}
@@ -704,11 +756,16 @@ extract_tiff (const gchar          *uri,
 	if (ed->y_resolution) {
 		gdouble value;
 
-		value = ed->resolution_unit == 1 ? g_strtod (ed->y_resolution, NULL) : g_strtod (ed->y_resolution, NULL) * CM_TO_INCH;
+		value = ed->resolution_unit != 3 ? g_strtod (ed->y_resolution, NULL) : g_strtod (ed->y_resolution, NULL) * CM_TO_INCH;
 		tracker_sparql_builder_predicate (metadata, "nfo:verticalResolution");
 		tracker_sparql_builder_object_double (metadata, value);
 	}
 
+	if (where != NULL) {
+		tracker_sparql_builder_append (metadata, where->str);
+		g_string_free (where, TRUE);
+	}
+
 	tiff_data_free (&td);
 	tracker_exif_free (ed);
 	tracker_xmp_free (xd);
