@@ -24,7 +24,7 @@ from common.utils.system import TrackerSystemAbstraction
 from common.utils.helpers import StoreHelper
 from common.utils import configuration as cfg
 from common.utils.storetest import CommonTrackerStoreTest as CommonTrackerStoreTest
-from common.utils.expectedFailure import expectedFailureBug as expectedFailureBug
+from common.utils.expectedFailure import expectedFailureBug, expectedFailureJournal
 import unittest2 as ut
 
 
@@ -178,9 +178,10 @@ class BackupRestoreTest (CommonTrackerStoreTest):
               instances_before = self.tracker.count_instances ("nco:Contact")
 	      self.tracker.backup (self.BACKUP_FILE)
 
+              self.system.tracker_store_stop_nicely ()
               self.system.tracker_store_remove_dbs ()
               self.system.tracker_store_remove_journal ()
-              self.system.tracker_store_brutal_restart ()
+              self.system.tracker_store_start ()
               
               instances_before_restore = self.tracker.count_instances ("nco:Contact")
               self.assertNotEqual (instances_before_restore, instances_before)
@@ -204,9 +205,10 @@ class BackupRestoreTest (CommonTrackerStoreTest):
               instances_before = self.tracker.count_instances ("nco:Contact")
 	      self.tracker.backup (self.BACKUP_FILE)
 
+              self.system.tracker_store_stop_brutally ()
               self.system.tracker_store_corrupt_dbs ()
               self.system.tracker_store_remove_journal ()
-              self.system.tracker_store_brutal_restart ()
+              self.system.tracker_store_start ()
               
               instances_before_restore = self.tracker.count_instances ("nco:Contact")
               self.assertNotEqual (instances_before_restore, instances_before)
@@ -217,89 +219,6 @@ class BackupRestoreTest (CommonTrackerStoreTest):
               # DB to the original state
               self.__delete_test_instance ()
               
-
-## 	def test_backup_09(self):
-## 	      """
-## 	      1.Insert a contact.
-## 	      2.Take backup of db.
-## 	      3.Corrupt the databse.
-## 	      4.Restore the db.
-## 	      5.Search for the contact inserted. 
-## 	      Expected:Contact should be listed in tracker search.	
-## 	      """
-
-## 	      urn = 'urn:uuid:'+`random.randint(0, sys.maxint)`
-## 	      insert_sparql = "INSERT { <%s> a nco:Contact } "
-## 	      insert = insert_sparql %(urn)
-## 	      self.sparql_update(insert)
-
-## 	      result = commands.getoutput('tracker-stats | grep nco:Contact')
-## 	      stats1 = result.split()
-
-## 	      self.backup.Save(BACKUP_1)
-
-##               delete_sparql = "DELETE { <%s> a nco:Contact } "                                                            
-##               delete = delete_sparql %(urn)                                                                               
-##               self.sparql_update(delete)  	
-		
-## 	      print ("Corrupting the database")
-
-## 	      commands.getoutput('cp  ' + configuration.TEST_DATA_IMAGES +'test-image-1.jpg' + ' ' + database +'meta.db' )
-## 	      try :
-##                  self.db_restore(BACKUP_1)
-## 	      except :
-## 		 print "Restore is not scuccessful"
-
-## 	      print "querying for the contact"
-## 	      query_sparql = "SElECT ?c WHERE {?c a nco:Contact .FILTER (REGEX(?c, '%s' )) }"
-##               query = query_sparql %(urn)                                             
-##               result = self.query(query)                                                     
-## 	      self.assert_(result[0][0].find(urn)!=-1 , "Restore is not successful" )
-
-## 	def test_backup_10(self):
-
-## 	      """
-## 	      1.Insert a contact.
-## 	      2.Take backup of db.
-## 	      3.Kill tracker-store process.
-## 	      4.Corrupt the databse.
-## 	      5.Restore the db.
-## 	      6.Search for the contact inserted. 
-## 	      Expected:Contact should be listed in tracker search.	
-## 	      """
-
-## 	      urn = random.randint(0, sys.maxint)	
-
-## 	      insert_sparql = "INSERT { <%s> a nco:Contact } "
-## 	      insert = insert_sparql %(urn)
-
-## 	      self.sparql_update(insert)
-
-## 	      result = commands.getoutput('tracker-stats | grep nco:Contact')
-## 	      stats1 = result.split()
-
-## 	      self.backup.Save(BACKUP_1)
-
-## 	      delete_sparql = "DELETE { <%s> a nco:Contact } "                                                            
-##               delete = delete_sparql %(urn)                                                                               
-##               self.sparql_update(delete)  
-
-## 	      self.kill_store() 
-
-## 	      print ("corrupting the database")
-## 	      commands.getoutput('cp  ' + configuration.TEST_DATA_IMAGES +'test-image-1.jpg' + ' ' + database +'meta.db' )
-
-## 	      try :
-##                  self.db_restore(BACKUP_1)
-## 	      except :
-## 		 print "Restore is not scuccessful"
-
-##               result = commands.getoutput('tracker-stats | grep nco:Contact ')
-## 	      query_sparql = "SElECT ?c WHERE {?c a nco:Contact .FILTER (REGEX(?c, '%s' )) }"
-##               query = query_sparql %(urn)                                                    
-##               result = self.query(query)                                                     
-## 	      self.assert_(result[0][0].find(urn)!=-1 , "Restore is not successful")
-
         def test_backup_11(self):
 	      """
               Backup ignores the file extension
@@ -334,14 +253,16 @@ class JournalReplayTest (CommonTrackerStoreTest):
         """
         Force journal replaying and check that the DB is correct aftewards
         """
+	@expectedFailureJournal()
  	def test_journal_01 (self) :
             """
             Journal replaying when the DB is corrupted
             
             Insert few data (to have more than the pre-defined instances)
             Check instances of different classes
+            Kill the store (SIGKILL)
             Replace the DB with a random file
-            Restart the daemon
+            Start the store
             Check instances of different classes
             """
             self.tracker.update ("INSERT { <test://journal-replay/01> a nco:Contact. }")
@@ -350,10 +271,13 @@ class JournalReplayTest (CommonTrackerStoreTest):
             ie = self.tracker.count_instances ("nie:InformationElement")
             contacts = self.tracker.count_instances ("nco:Contact")
 
+            self.system.tracker_store_stop_brutally ()
             self.system.tracker_store_corrupt_dbs ()
-            self.system.tracker_store_brutal_restart ()
+            self.system.tracker_store_start ()
             ## Start it twice... the first time it detects the broken DB and aborts
-            self.system.tracker_store_brutal_restart ()
+            #self.system.tracker_store_start ()
+
+            self.tracker.connect ()
 
             emails_now = self.tracker.count_instances ("nmo:Email")
             ie_now = self.tracker.count_instances ("nie:InformationElement")
@@ -365,14 +289,16 @@ class JournalReplayTest (CommonTrackerStoreTest):
 
             self.tracker.update ("DELETE { <test://journal-replay/01> a rdfs:Resource. }")
 
+	@expectedFailureJournal()
 	def test_journal_02 (self) :
             """
             Journal replaying when the DB disappears
             
             Insert few data (to have more than the pre-defined instances)
             Check instances of different classes
-            Remove the DB
-            Restart the daemon
+            KILL the store
+            Force a journal replay (DB missing, .meta.isrunning exists, journal OK)
+            Start the store
             Check instances of different classes
             """
             self.tracker.update ("INSERT { <test://journal-replay/02> a nco:Contact. }")
@@ -381,8 +307,11 @@ class JournalReplayTest (CommonTrackerStoreTest):
             ie = self.tracker.count_instances ("nie:InformationElement")
             contacts = self.tracker.count_instances ("nco:Contact")
 
-            self.system.tracker_store_remove_dbs ()
-            self.system.tracker_store_brutal_restart ()
+            self.system.tracker_store_stop_brutally ()
+            self.system.tracker_store_prepare_journal_replay ()
+            self.system.tracker_store_start ()
+
+            self.tracker.connect ()
 
             emails_now = self.tracker.count_instances ("nmo:Email")
             ie_now = self.tracker.count_instances ("nie:InformationElement")
